﻿using System;
using System.Data;
using System.Data.OleDb;
using System.Timers;


namespace PhonicsClassLibrary
{
    public interface IChamada
    {
        /// <summary>
        /// Método para registar uma chamada de um cartão
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartao</param>
        /// <param name="numero">Número de destino<param</param>
        /// <param name="datainicial">Data em que a chamada foi iniciada</param>
        /// <param name="datafinal">Data em que a chamada foi finalizada</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos, o código do cartão válido e os dados da chamada</pre>
        /// <post>Regista uma nova chamada na base de dados</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Registar(string username, string pass,
            int cartaoID, int numero, DateTime datainicial, DateTime datafinal);

        /// <summary>
        /// Método para registar o inicio de uma chamada de um cartão
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="cartaoID">Código do cartao</param>
        /// <param name="numeroorigem">Número de origem<param>
        /// <param name="numerodestino">Número de destino<param>
        /// <param name="data">Data do inicio da chamada</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos, o código do cartão válido e os dados do inicio da
        /// chamada</pre>
        /// <post>Regista o inicio de uma nova chamada</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Iniciar(string username, string pass,
            int cartaoID, int numeroorigem, int numerodestino, DateTime data);

        /// <summary>
        /// Método para registar o fim de uma chamada de um cartão
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="numeroorigem">Número de origem</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o número de origem da chamada</pre>
        /// <post>Regista o fim  de uma chamada</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Finalizar(string username, string pass,
            int numeroorigem);


        /// <summary>
        /// Método para procurar as chamadas que estão activas
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <pre>Este método recebe o username e password do utilizador 
        /// válidos</pre>
        /// <post>É retornada toda a informação das chamadas activas</post>
        /// <returns>Retorna o estado da operação e o dataset com
        /// as chamadas acticas</returns>
        PhonicsStatusEnum GetChamadasActivas(string username, string pass,
            out DataSet chamadasActivas);

        /// <summary>
        /// Método para retornar os segundos de uma chamada activa de
        /// um determinado numero
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="numeroorigem">Número de origem</param>
        /// <pre>Este método recebe o username e password do utilizador 
        /// válidos</pre>
        /// <post>É retornado os segundos de uma determinada chamada
        /// activa</post>
        /// <returns>Retorna o estado da operação e o numero de
        /// segundos</returns>
        PhonicsStatusEnum GetSegundos(string username, string pass,
            int numeroorigem, out int nsegundos);

        /// <summary>
        /// Método para obter informações de uma chamada através do seu
        /// código
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="chamadaID">Código da chamada</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válido e o código da chamada válido</pre>
        /// <post>É retornada toda a informação da chamada com o código
        /// da chamada recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações da chamada
        /// </returns>
        DataSet GetInfoByID(string username, string pass, int chamadaID);

    }

    
    internal class Chamada : IChamada
    {
        public Chamada()
        {
        }

        // possui estado
        Timer myTimer = null;
        string CONN = @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=D:\ISEP\Adav\P\Phonics\dadosPhonics.mdb";


        // Processo de incremento de tempo nas chamadas activas
        private void myTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            OleDbConnection conn = new OleDbConnection(CONN);
            int n = 0;

            // Actualiza todas as chamadas
            try
            {
                conn.Open();

                
                OleDbCommand cmd = new OleDbCommand("UPDATE Chamada SET duracao = duracao + 1 WHERE activa = Yes", conn);
                n = cmd.ExecuteNonQuery();            
                
            }
            catch
            {
                myTimer.Enabled = false;
                myTimer = null;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            // Pára actualizações
            if (n == 0)
            {
                myTimer.Enabled = false;
                myTimer = null;
            }
        }





        public PhonicsStatusEnum Registar(string username, string pass,
            int cartaoID, int numero, DateTime datainicial, DateTime datafinal)
        {  


            PhonicsStatusEnum status = PhonicsStatusEnum.OK;
            DataSet dsc = null; 
            DataSet dst = null;
            OleDbConnection conn = null;
            string dia="";

            //
            // validar o utilizador
            //
            if (LigaDB.Validar(username, pass) != PhonicsStatusEnum.OK)
            {
                return PhonicsStatusEnum.INVALID_LOGIN;
            }

            try
            {
                //
                // criar objecto DataSet
                //
                dsc = new DataSet(); 
                dst = new DataSet();

                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //SELECIONAR TODOS OS CAMPOS DO CARTÃO
                string sSqlCmd = "Select * From Cartao Where cartaoID=?";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Parameters.AddWithValue("cartaoID", cartaoID);

                oAdapter.Fill(dsc, "Cartao");

              
                int tarifarioID = (int)dsc.Tables["Cartao"].Rows[0]["tarifarioID"];
                int numeroorigem = (int)dsc.Tables["Cartao"].Rows[0]["numero"];

                //CLACULAR A DURAÇÃO DA CHAMADA
                TimeSpan dDif =  datafinal.Subtract(datainicial);               
                long duracao = ((long)dDif.TotalSeconds)+1;
        

                //VERIFICAR SE A CHAMADA É FEITA DURANTE A SEMANA(DAS) OU 
                //DURANTE O FIM DE SEMANA(FDS)
                if (datainicial.DayOfWeek == DayOfWeek.Monday)
                    dia = "DAS";
                else
                    if (datainicial.DayOfWeek == DayOfWeek.Tuesday)
                        dia = "DAS";
                    else
                        if (datainicial.DayOfWeek == DayOfWeek.Wednesday)
                            dia = "DAS";
                        else
                            if (datainicial.DayOfWeek == DayOfWeek.Thursday)
                                dia = "DAS";
                            else
                                if (datainicial.DayOfWeek == DayOfWeek.Friday)
                                    dia = "DAS";
                                else
                                    if (datainicial.DayOfWeek == DayOfWeek.Saturday)
                                        dia = "FDS";
                                    else
                                        if (datainicial.DayOfWeek == DayOfWeek.Sunday)
                                            dia = "FDS";
               

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                //DETERMINAR QUAL É A LINHA DE DETALHE DO TARIFARIO RELATIVA 
                //AO MOMENTO EM QUE A CHAMADA FOI REALIZADA
                //DE MANEIRA A SABERMOS O PERIODO E O CUSTO DO TARIFARIO
                sSqlCmd = "Select * From TarifarioDetalhe Where tarifarioID=? AND rede=left(?,2) AND dia=? AND right(inicio,8)<? AND right(fim,8)>?";
                oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Parameters.AddWithValue("tarifarioID", tarifarioID);
                oAdapter.SelectCommand.Parameters.AddWithValue("rede", numero);
                oAdapter.SelectCommand.Parameters.AddWithValue("dia", dia);
                oAdapter.SelectCommand.Parameters.AddWithValue("inicio", datainicial.TimeOfDay.ToString());
                oAdapter.SelectCommand.Parameters.AddWithValue("fim", datainicial.TimeOfDay.ToString());
                
                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(dst, "TarifarioDetalhe");

                int periodo = (int)dst.Tables["TarifarioDetalhe"].Rows[0]["Periodo"];
                Decimal custo = (Decimal)dst.Tables["TarifarioDetalhe"].Rows[0]["Custo"];


                //CALCULAR O CUSTO DA CHAMADA
                decimal num = duracao / (decimal)periodo;
                int dur = (int)Math.Ceiling(num);
                decimal custofinal =  (decimal)dur* custo;


                //REGISTAR NA BASE DE DADOS A CHAMADA
                OleDbCommand Cmd = new OleDbCommand("Insert into Chamada (CartaoID, numeroorigem, numerodestino, data, duracao, custo, activa) Values (?, ?, ?, ?, ?, ?, No)", conn);
                Cmd.Parameters.AddWithValue("cartaoID", cartaoID);
                Cmd.Parameters.AddWithValue("numeroorigem", numeroorigem);
                Cmd.Parameters.AddWithValue("numerodestino", numero);
                Cmd.Parameters.AddWithValue("data", datafinal.ToString());
                Cmd.Parameters.AddWithValue("duracao", duracao);
                Cmd.Parameters.AddWithValue("custo", custofinal);

                int insRows = Cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;

                //CUSTO FINAL A DESCONTAR AO SALDO DO CARTAO
                float c = (float)custofinal;

                //ACTUALIZA O SALDO DO CARTAO
                Cmd = new OleDbCommand("Update Cartao Set Saldo=Saldo-? WHERE cartaoID=?", conn);
                Cmd.Parameters.AddWithValue("Saldo", c);
                Cmd.Parameters.AddWithValue("cartaoID", cartaoID);

                int insRows2 = Cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows2 == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;             
            }

            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método 'Registar': " + ex);
                status = PhonicsStatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return status;


        }

        public PhonicsStatusEnum Iniciar(string username, string pass,
            int cartaoID, int numeroorigem, int numerodestino, DateTime data)
        {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;

            OleDbConnection conn = null;

            if ((numeroorigem == 0) ||
               (numerodestino == 0) ||
               (data == null)) return PhonicsStatusEnum.PARAMETER_PROBLEM;


            conn = new OleDbConnection(LigaDB.CONN);

            // Chamada já existente?
            try
            {
                conn.Open();
                //SELECIONAR UMA CHAMADA DE MANEIRA A SABER SE ELA JA ESTA ACTIVA OU NÃO
                OleDbCommand cmd = new OleDbCommand("SELECT activa FROM Chamada WHERE activa = Yes AND (numeroorigem = ? OR numeroorigem = ? OR numerodestino = ? OR numerodestino = ? )", conn);
                cmd.Parameters.AddWithValue("origem1", numeroorigem);
                cmd.Parameters.AddWithValue("origem2", numerodestino);
                cmd.Parameters.AddWithValue("destino1", numerodestino);
                cmd.Parameters.AddWithValue("destino2", numeroorigem);
                object obj = cmd.ExecuteScalar();
                if (obj != null) return PhonicsStatusEnum.ALREADY_INITIATED;

                //REGISTAR UM INICIO DE UMA CHAMADA
                cmd = new OleDbCommand("INSERT INTO Chamada (cartaoID,numeroorigem, numerodestino,data, duracao, custo ,activa) VALUES (?,?,?,?,0,0,Yes)", conn);
                cmd.Parameters.AddWithValue("cartaoID", cartaoID);
                cmd.Parameters.AddWithValue("numeroorigem", numeroorigem);
                cmd.Parameters.AddWithValue("numerodestino", numerodestino);
                cmd.Parameters.AddWithValue("@data", data).OleDbType = OleDbType.Date;
                

                cmd.ExecuteNonQuery();
            }
            catch
            {
                return PhonicsStatusEnum.DATABASE_PROBLEM;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            // Inicia o processo de Timer se nunca foi chamado      
            
            
            if (myTimer == null)
            {
                myTimer = new Timer(1000); // 1 segundo
                myTimer.AutoReset = true;  // reset the timer
                myTimer.Elapsed +=          // regista handler
                new System.Timers.ElapsedEventHandler(myTimer_Elapsed);
                myTimer.Start();
            }

            return PhonicsStatusEnum.OK;
        }

        public PhonicsStatusEnum Finalizar(string username, string pass,
            int numeroorigem)
        {
            PhonicsStatusEnum status;

            DataSet ds = null;
            DataSet ds2=null;

            string dia = "";

            if ((numeroorigem == 0))return PhonicsStatusEnum.PARAMETER_PROBLEM;
            OleDbConnection conn = new OleDbConnection(LigaDB.CONN);

            // Chamada já existente?
            try
            {
                ds = new DataSet();
                ds2=new DataSet();

                conn.Open();
              
                //DETERMINAR A CHAMADA QUE ESTA ACTIVA QUE FOI REALIZADA DO NUMERO
                //DE MANEIRA A SE SABER A DURACAO, ID DO CARTAO, DATA E NUMERO
                //DE DESTINO DESSA CHAMADA
                string sSqlCmd = "Select * From Chamada Where activa=yes AND numeroorigem=?";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Parameters.AddWithValue("numeroorigem", numeroorigem);

                oAdapter.Fill(ds, "Chamada");

                int duracao = (int)ds.Tables["Chamada"].Rows[0]["duracao"];
                int cartaoID= (int)ds.Tables["Chamada"].Rows[0]["cartaoID"];
                DateTime inicio = (DateTime)ds.Tables["Chamada"].Rows[0]["data"];
                int numerodestino = (int)ds.Tables["Chamada"].Rows[0]["numerodestino"];


                //VERIFICAR SE A CHAMADA É FEITA DURANTE A SEMANA(DAS) OU 
                //DURANTE O FIM DE SEMANA(FDS)
                if (inicio.DayOfWeek == DayOfWeek.Monday)
                    dia = "DAS";
                else
                    if (inicio.DayOfWeek == DayOfWeek.Tuesday)
                        dia = "DAS";
                    else
                        if (inicio.DayOfWeek == DayOfWeek.Wednesday)
                            dia = "DAS";
                        else
                            if (inicio.DayOfWeek == DayOfWeek.Thursday)
                                dia = "DAS";
                            else
                                if (inicio.DayOfWeek == DayOfWeek.Friday)
                                    dia = "DAS";
                                else
                                    if (inicio.DayOfWeek == DayOfWeek.Saturday)
                                        dia = "FDS";
                                    else
                                        if (inicio.DayOfWeek == DayOfWeek.Sunday)
                                            dia = "FDS";




                //DETERMINAR QUAL O ID DO TARIFARIO DO CARTAO DO QUAL FOI FEITA A CHAMADA
                OleDbCommand cmd = new OleDbCommand("Select tarifarioID From Cartao Where cartaoID=?", conn);
                cmd.Parameters.AddWithValue("cartaoID", cartaoID);

                int tarifarioID = (int)cmd.ExecuteScalar();


                //SELECIONAR A LINHA DE DETALHE CORRESPONDENTE AO MOMENTO EM QUE A CHAMADA FOI REALIZADA
                sSqlCmd = "Select * From TarifarioDetalhe Where tarifarioID=? AND rede=left(?,2) AND dia=? AND right(inicio,8)<? AND right(fim,8)>?";
                oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Parameters.AddWithValue("tarifarioID", tarifarioID);
                oAdapter.SelectCommand.Parameters.AddWithValue("rede", numerodestino);
                oAdapter.SelectCommand.Parameters.AddWithValue("dia", dia);
                oAdapter.SelectCommand.Parameters.AddWithValue("inicio", inicio.TimeOfDay.ToString());
                oAdapter.SelectCommand.Parameters.AddWithValue("fim", inicio.TimeOfDay.ToString());//INICIO OU FIM??

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds2, "TarifarioDetalhe");


                int periodo = (int)ds2.Tables["TarifarioDetalhe"].Rows[0]["Periodo"];
                Decimal custo = (Decimal)ds2.Tables["TarifarioDetalhe"].Rows[0]["Custo"];//MUDAR TIPO DO CUSTO NO ACCESS



                //CALUCLAR O CUSTO DA CHAMADA
                decimal num = duracao / (decimal)periodo;
                int dur = (int)Math.Ceiling(num);
                decimal custofinal = (decimal)dur * custo;


                //CUSTO FINAL A DESCONTAR AO SALDO
                float c = (float)custofinal;

                //ACTUALIZA O SALDO DO CARTAO
                OleDbCommand Cmd = new OleDbCommand("Update Cartao Set Saldo=Saldo-? WHERE cartaoID=?", conn);
                Cmd.Parameters.AddWithValue("Saldo", c);
                Cmd.Parameters.AddWithValue("cartaoID", cartaoID);

                int insRows2 = Cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows2 == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;


                


                //ALTERA O ESTADO DA CHAMADA PARA INACTIVA E GUARDA O CUSTO DA CHAMADA
                cmd = new OleDbCommand("UPDATE Chamada SET activa = No, custo=? WHERE numeroorigem= ? AND activa = Yes", conn);
                cmd.Parameters.AddWithValue("custo", custofinal);
                cmd.Parameters.AddWithValue("numeroorigem", numeroorigem);
                

                int n = cmd.ExecuteNonQuery();
                if (n < 1) return PhonicsStatusEnum.NOT_INITIATED;
                if (n > 1) return PhonicsStatusEnum.MULTIPLE_CALLS;
            }
            catch
            {
                return PhonicsStatusEnum.DATABASE_PROBLEM;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            return PhonicsStatusEnum.OK;
        }



        public PhonicsStatusEnum GetChamadasActivas(string username, string pass, 
            out DataSet chamadasActivas)
        {
            PhonicsStatusEnum status;
            OleDbConnection conn = null;
            chamadasActivas = new DataSet();
            OleDbDataAdapter da = new OleDbDataAdapter("SELECT numeroorigem, numerodestino, data, duracao FROM Chamada WHERE activa = Yes", CONN);
            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();


                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return PhonicsStatusEnum.INVALID_LOGIN;
                
                
                da.Fill(chamadasActivas);
            }
            catch
            {
                return  PhonicsStatusEnum.DATABASE_PROBLEM;
            }

            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return  PhonicsStatusEnum.OK;
        }

        public PhonicsStatusEnum GetSegundos(string username, string pass, 
            int numeroorigem, out int nsegundos)
        {
            PhonicsStatusEnum status;
            nsegundos = -1;
            if ((numeroorigem == 0)) return PhonicsStatusEnum.PARAMETER_PROBLEM;

            OleDbConnection conn = new OleDbConnection(LigaDB.CONN);

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //

                conn.Open();

                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return PhonicsStatusEnum.INVALID_LOGIN;

             
                OleDbCommand cmd = new OleDbCommand("SELECT duracao FROM Chamada WHERE numeroorigem = ? AND activa = Yes", conn);
                cmd.Parameters.AddWithValue("numeroorigem", numeroorigem);

                object obj = cmd.ExecuteScalar();
                if (obj == null) return PhonicsStatusEnum.INEXISTENT_INITIATOR;
                nsegundos = (int)obj;
            }
            catch
            {
                return PhonicsStatusEnum.DATABASE_PROBLEM;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            return PhonicsStatusEnum.OK;
        }


        public DataSet GetInfoByID(string username, string pass,
           int chamadaID)
        {
            OleDbConnection conn = null;
            DataSet ds = null;

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar utilizar
                //
                PhonicsStatusEnum status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return null;

                //
                // efectuar pesquisa
                //
                ds = LigaDB.GetById(conn, null, "Chamada", "ChamadaID", chamadaID);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <GetByID> da classe 'Chamada': " + ex.Message);
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }

    }



}