﻿
using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;




namespace PhonicsClassLibrary
{
    internal class LigaDB
    {
		private const string DEFAULT_CONN = @"Provider=Microsoft.Jet.OLEDB.4.0;User ID=;Password=;Mode=Share Deny None;Data Source=";

		public static string CONN
		{
			get 
			{
                string dbPath =	System.Configuration.ConfigurationManager.AppSettings["DBPath"];
               

				if (dbPath == null || dbPath.Length == 0)
				{                                                             
					throw new ApplicationException("Falta configurar o caminho para a BD ''");
				}

				return DEFAULT_CONN + dbPath; 
			}
		}

		private LigaDB()
		{
		}

		#region Validar

		/// <summary>
		/// Valida um login/password no sistema
		/// </summary>
		/// <param name="User">username de login na aplicação</param>
		/// <param name="Pass">password de login na aplicação</param>
		/// <returns></returns>
		static public PhonicsStatusEnum Validar(string username, string pass)
		{
			PhonicsStatusEnum status = PhonicsStatusEnum.OK;
			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto de conexão à base de dados e abrir a conexão
				//
				conn = new OleDbConnection(CONN);
				conn.Open();

				//
				// invocar método auxiliar
				//
				status = Validar(conn, null, username, pass);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO na validação do 'Utilizador': " + ex);
				status =PhonicsStatusEnum.ERROR;
			}
			finally
			{
				//
				// fechar a conexão
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		/// <summary>
		/// Valida um login/password no sistema
		/// </summary>
		/// <param name="User">username de login na aplicação</param>
		/// <param name="Pass">password de login na aplicação</param>
		/// <returns></returns>
		static public PhonicsStatusEnum Validar(OleDbConnection conn, OleDbTransaction tx, string username, string pass)
		{
			PhonicsStatusEnum status = PhonicsStatusEnum.OK;
			OleDbCommand    cmd  = null;

			try
			{
			
                cmd = new OleDbCommand("Select * from Utilizador Where [Username]=? and Pass=?", conn); //[User] pq é uma palavra reservada do Access
                cmd.Parameters.AddWithValue("Username", username);
                cmd.Parameters.AddWithValue("Password", pass);
                cmd.Transaction = tx;


                

				//
				// executar o comando e obter um DataReader
				//
				OleDbDataReader dr = cmd.ExecuteReader();
                
				//
				// verificar se retornou dados 
				//
				if (dr.HasRows)
					status = PhonicsStatusEnum.OK;
				else
					status = PhonicsStatusEnum.INVALID_LOGIN;

				dr.Close();
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO na validação do 'Username': " + ex);
				status = PhonicsStatusEnum.ERROR;
			}

			return status;
		}

        /// <summary>
        /// Valida um login/password no sistema e determina se é Supervisor ou Operador
        /// </summary>
        /// <param name="User">username de login na aplicação</param>
        /// <param name="Pass">password de login na aplicação</param>
        /// <returns></returns>
        static public PhonicsStatusEnum ValidarComTipo(OleDbConnection conn, OleDbTransaction tx, string username, string pass)
        {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;
            OleDbCommand cmd = null;
            string tipo = "Supervisor";

            try
            {

                cmd = new OleDbCommand("Select * from Utilizador Where [Username]=? and Pass=? and tipo=?", conn); //[User] pq é uma palavra reservada do Access
                cmd.Parameters.AddWithValue("Username", username);
                cmd.Parameters.AddWithValue("Password", pass);
                cmd.Parameters.AddWithValue("tipo", tipo);
                cmd.Transaction = tx;
                
                //
                // executar o comando e obter um DataReader
                //
                OleDbDataReader dr = cmd.ExecuteReader();

                //
                // verificar se retornou dados 
                //
                if (dr.HasRows)
                    status = PhonicsStatusEnum.OK;
                else
                    status = PhonicsStatusEnum.INVALID_LOGIN;

                dr.Close();
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO na validação do 'Username': " + ex);
                status = PhonicsStatusEnum.ERROR;
            }

            return status;
        }



		#endregion




        #region GetById

        /// <summary>
        /// Obtem um dataset resultante de uma pesquisa por chave
        /// </summary>
        /// <param name="table">tabela a pesquisar</param>
        /// <param name="field">campo chave a usar</param>
        /// <param name="key">valor da chave a pesquisar</param>
        /// <returns>um dataset do tipo pedido</returns>
        static public DataSet GetById(string table, string keyName, int key)
        {
            DataSet ds = null;
            OleDbConnection conn = null;

            try
            {
                //
                // criar objecto DataSet
                //
                ds = new DataSet();

                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(CONN);
                conn.Open();

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                //
                string sSqlCmd = "Select * From " + table + " Where " + keyName + " = " + key.ToString();
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, table);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método 'LigaDB.GetByID': " + ex.Message);
                ds = null;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }

        /// <summary>
        /// Obtem um dataset resultante de uma pesquisa por chave
        /// </summary>
        /// <param name="conn">a conexão a utilizar</param>
        /// <param name="table">tabela a pesquisar</param>
        /// <param name="keyName">campo chave a usar</param>
        /// <param name="key">valor da chave a pesquisar</param>
        /// <returns>um dataset do tipo pedido</returns>
        static public DataSet GetById(OleDbConnection conn, OleDbTransaction tx, string table, string keyName, int key)
        {
            DataSet ds = null;

            try
            {
                //
                // criar objecto DataSet
                //
                ds = new DataSet();

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                //
                string sSqlCmd = "Select * From " + table + " Where " + keyName + " = " + key.ToString();
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Transaction = tx;

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, table);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método 'LigaDB.GetByID': " + ex.Message);
                ds = null;
            }

            return ds;
        }

        /// <summary>
        /// Obtem um dataset resultante de uma pesquisa por chave
        /// </summary>
        /// <param name="table">tabela a pesquisar</param>
        /// <param name="field">campo chave a usar</param>
        /// <param name="key">valor da chave a pesquisar</param>
        /// <returns>um dataset do tipo pedido</returns>
        static public DataSet GetById(OleDbConnection conn, OleDbTransaction tx, string table, string keyName, int[] keys)
        {
            DataSet ds = null;
            try
            {
                //
                // criar objecto DataSet
                //
                ds = new DataSet();

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                //
                System.Text.StringBuilder sb = new System.Text.StringBuilder("Select * From " + table + " Where " + keyName + " IN (");
                foreach (int k in keys)
                {
                    sb.Append(k.ToString());
                    sb.Append(",");
                }
                sb.Length = sb.Length - 1;
                sb.Append(")");
                string sSqlCmd = sb.ToString();

                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Transaction = tx;

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, table);
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método 'LigaDB.GetByID': " + ex.Message);
                ds = null;
            }

            return ds;
        }

        #endregion




        #region FindByStringField

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		static public DataSet FindByStringField(string table, string field, string key)
		{
			OleDbConnection conn = null;
			DataSet ret = new DataSet();

			try
			{
				//
				// criar objecto de conexão à base de dados
				//
				conn = new OleDbConnection(LigaDB.CONN);

				//
				// invocar método auxiliar
				//
				ret = FindByStringField(conn, table, field, key);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO no componente 'LigaDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexão
				//
				if (conn != null && conn.State != ConnectionState.Closed)
					conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		static public DataSet FindByStringField(OleDbConnection conn, string table, string field, string key)
		{
			DataSet ret = new DataSet();

			try
			{
				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM " + table + " WHERE " + field + " LIKE ?", conn);
				da.SelectCommand.Parameters.AddWithValue("key", key+"%");

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret, table);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO no componente 'LigaDB': " + ex);
			}

			//
			// retornar valor
			//
			return ret;
		}

		#endregion
	}
}





