﻿using System;
using System.Data;
using System.Data.OleDb;


namespace PhonicsClassLibrary
{
    public interface ITarifario
    {
        /// <summary>
        /// Método que insere um novo tarifário
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="descricao">Descrição do tarifário</param>
        /// <param name="comissao">Comissao que recebe cada operador</param>
        /// <param name="tarifariodetalhes">Detalhes do tarifario</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e os dados do novo tarifário</pre>
        /// <post>Insere um novo tarifário na base de dados</post>
        /// <returns>Retorna o estado da operação</returns>
        /// 
        /// --------> periodo e comissao, retorna ID,, retirei rede
        PhonicsStatusEnum Inserir(string username, string pass,
           string descricao, float comissao, DataSet tarifariodetalhes);

        /// <summary>
        /// Método para actualizar um tarifário
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="tarifarioID">Código do tarifário</param>
        /// <param name="descricao">Descrição do tarifário</param>
        /// <param name="rede">Redes às quais se destina o tarifário
        /// </param>
        /// <param name="dia">Dias que pertencem ao tarifário</param>
        /// <param name="inicio">Hora inicial do tarifário</param>
        /// <param name="fim">Hora final do tarifário</param>
        /// <param name="custo">Custo de cada período</param>
        /// <param name="periodo">Periodo de facturação</param>
        /// <param name="comissao">Comissão que recebe cada operador</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o código do tarifário válido e os dados do
        /// tarifário</pre>
        /// <post>Actualiza informação do tarifário</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Update(string username, string pass, 
            int tarifarioID, string descricao, RedeEnum rede,
            DiasEnum dia, DateTime inicio, DateTime fim, float custo,
            int periodo, float comissao);

        /// <summary>
        /// Método para obter informações de um tarifario através do seu
        /// código
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="tarifarioID">Código do tarifario</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos e o código do tarifario válido</pre>
        /// <post>É retornada toda a informação do tarifario com o código
        /// de tarifario recebido como parametro</post>
        /// <returns>Retorna um dataset com as informações do tarifario
        /// </returns>
        DataSet GetInfoByID(string username, string pass, int tarifarioID);

        /// <summary>
        /// Método para obter informações de todos os tarifarios</summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos</pre>
        /// <post>É retornada toda a informação de todos os tarifarios</post>
        /// <returns>Retorna um dataset com as informações dos tarifarios
        /// </returns>
        DataSet GetAll(string username, string pass);

        /// <summary>
        /// Método para criar linhas de detalhe de um tarifario
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos</pre>
        /// <post>É retornada toda a informação das linhas de detalhe de
        /// um tarifario</post>
        /// <returns>Retorna um dataset com as linhas de detalhe do tarifario
        /// </returns>
        DataSet CriarDetalhes(string username, string pass);
    }


    internal class Tarifario : ITarifario
    {
        public Tarifario()
        {
        }
        public PhonicsStatusEnum Inserir(string username, string pass,
            string descricao,float comissao, DataSet tarifariodetalhe)

        {

            OleDbConnection conn = null;
            OleDbTransaction tx = null;
            PhonicsStatusEnum ret = PhonicsStatusEnum.OK;

            try
            {
                //
                // criar objecto de conexão à base de dados
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // iniciar transacção
                //
                tx = conn.BeginTransaction();

                //
                // validar o utilizador
                //
                if (LigaDB.ValidarComTipo(conn, tx, username, pass) != PhonicsStatusEnum.OK)
                    return PhonicsStatusEnum.INVALID_LOGIN;

          
                //
                // criar comando para inserção
                //
                //INSERIR UM NOVO TARIFARIO NA BASE DE DADOS
                OleDbCommand cmd = new OleDbCommand("INSERT INTO Tarifario (descricao, comissao) VALUES (?, ?)", conn);
                cmd.Transaction = tx;
                cmd.Parameters.AddWithValue("descricao", descricao);
                cmd.Parameters.AddWithValue("comissao", comissao);

                //
                // inserir registo 
                //
                cmd.ExecuteNonQuery();

                //
                // obter id autogerado
                //
                OleDbCommand cmd2 = new OleDbCommand("SELECT @@IDENTITY", conn);
                cmd2.Transaction = tx;
                int id = (int)cmd2.ExecuteScalar();

                //
                // inserir chave estrangeira nas linhas de detalhe
                //
                foreach (DataRow regDetails in tarifariodetalhe.Tables["TarifarioDetalhe"].Rows)
                {
                    regDetails["tarifarioID"] = id;
                }

                //
                // inserir linhas de detalhe
                //
                OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM TarifarioDetalhe", conn);
                da.SelectCommand.Transaction = tx;
                OleDbCommandBuilder cb = new OleDbCommandBuilder(da);
                da.Update(tarifariodetalhe, "TarifarioDetalhe");

               
                //
                // fechar transacção
                //
                tx.Commit();
            }
            catch (OleDbException ex)
            {
                //
                // descartar mudanças efectuadas
                //
                if (tx != null)
                    tx.Rollback();

                //
                // tratar a excepção!!!!
                //
                ret = PhonicsStatusEnum.DATABASE_PROBLEM;
            }
            finally
            {
                conn.Close();
            }

            return ret;
        }

          public PhonicsStatusEnum Update(string username, string pass, 
            int tarifarioID, string descricao, RedeEnum rede,
            DiasEnum dia, DateTime inicio, DateTime fim, float custo, int periodo,
            float comissao)
             {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;

            OleDbConnection conn = null;

       
            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = LigaDB.ValidarComTipo(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return PhonicsStatusEnum.INVALID_LOGIN;

                //
                // criar comando SQL 1 a executar
                //
                //ACTUALIZAR OS DADOS DO TARIFARIO  ****NÃO ESTA A FUNCIONAR****
                string sqlCmd1 = "UPDATE Tarifario SET Descricao=?, Comissao=? WHERE tarifarioID=?";

                OleDbCommand cmd1 = new OleDbCommand(sqlCmd1, conn);

                if (descricao == null || descricao.Length == 0)
                    cmd1.Parameters.AddWithValue("descricao", DBNull.Value);
                else
                    cmd1.Parameters.AddWithValue("descricao", descricao);

                if (comissao == 0)
                    cmd1.Parameters.AddWithValue("comissao", DBNull.Value);
                else
                    cmd1.Parameters.AddWithValue("comisaao", comissao);

                cmd1.Parameters.AddWithValue("tarifarioID", tarifarioID);

                //
                // executar o comando 
                //
                int insRows = cmd1.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.DATABASE_PROBLEM;



                //
                // criar comando SQL 2 a executar
                //
                string sqlCmd2 = "UPDATE TarifarioDetalhe SET Rede=?, Dia=?, Inicio=?, Fim=?, Custo=?, Periodo=? WHERE tarifarioID=?";

                OleDbCommand cmd2 = new OleDbCommand(sqlCmd2, conn);

                //cmd.Parameters.AddWithValue("Morada", morada);

                if (rede == 0 || rede == 0)
                    cmd2.Parameters.AddWithValue("rede", DBNull.Value);
                else
                    cmd2.Parameters.AddWithValue("rede", rede);

                if (dia == 0 || dia == 0)
                    cmd2.Parameters.AddWithValue("dia", DBNull.Value);
                else
                    cmd2.Parameters.AddWithValue("dia", dia);

                if (inicio == null || inicio == null)
                    cmd2.Parameters.AddWithValue("inicio", DBNull.Value);
                else
                    cmd2.Parameters.AddWithValue("inicio", inicio);

                if (fim == null || fim == null)
                    cmd2.Parameters.AddWithValue("fim", DBNull.Value);
                else
                    cmd2.Parameters.AddWithValue("fim", fim);

                if (custo == 0 || custo == 0)
                    cmd2.Parameters.AddWithValue("custo", DBNull.Value);
                else
                    cmd2.Parameters.AddWithValue("custo", custo);

                if (periodo == 0 || periodo == 0)
                    cmd2.Parameters.AddWithValue("periodo", DBNull.Value);
                else
                    cmd2.Parameters.AddWithValue("periodo", periodo);

                
                cmd2.Parameters.AddWithValue("tarifarioID", tarifarioID);

                //
                // executar o comando 
                //
                int insRows2 = cmd2.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows2 == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.DATABASE_PROBLEM;

            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <Update> da classe 'Tarifario': " + ex.Message);
                status = PhonicsStatusEnum.DATABASE_PROBLEM;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }
            return status;

        }

        public DataSet GetInfoByID(string username, string pass,
           int tarifarioID)
        {
            OleDbConnection conn = null;
            DataSet ds = new DataSet(); ;

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar utilizar
                //
                PhonicsStatusEnum status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return null;

                //
                // efectuar pesquisa
                //
                //SELECIONA TODA A INFORMAÇÃO RELATIVAMENTE AO TARIFARIO
                string sSqlCmd = "Select * From ConsultaTarifario Where tarifarioID = ?";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Parameters.AddWithValue("tarifarioID", tarifarioID);



                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, "TarifarioDetalhe");



            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <GetByID> da classe 'Cliente': " + ex.Message);
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }

        public DataSet GetAll(string Username, string Pass)
        {
            //
            // validar o utilizador
            //
            if (LigaDB.Validar(Username, Pass) != PhonicsStatusEnum.OK)
            {
                return null;
            }

            DataSet ds = null;
            OleDbConnection conn = null;

            try
            {
                //
                // criar objecto DataSet
                //
                ds = new DataSet();

                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // criar um Datadapter para executar o comando e devolver o dataset
                //
                //SELECIONA TODA A INFORMAÇÃO RELATIVAMENTE A TODOS OS TARIFARIOS
                string sSqlCmd = "Select * From ConsultaTarifario";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

                //
                // executar o comando e preencher um Dataset
                //
                oAdapter.Fill(ds, "Tarifario");

                
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método 'GetAll': " + ex);
                ds = null;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;

        }

        public DataSet CriarDetalhes(string Username, string Pass)
        {
            OleDbConnection conn = null;
            DataSet ds;

            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                if (LigaDB.Validar(conn, null, Username, Pass) != PhonicsStatusEnum.OK)
                    return null;

                //
                // criar o dataset
                //
                ds = LigaDB.GetById(conn, null, "TarifarioDetalhe", "TarifarioID", -1); ;
                if (ds == null)
                    return null;

                //
                // configurar as colunas para utilização
                //
                ds.Tables["TarifarioDetalhe"].Columns["TarifarioID"].AllowDBNull = true;
            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <CriarDetalhe> da classe 'Tarifario': " + ex.Message);
                return null;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn.State == ConnectionState.Open)
                    conn.Close();
            }

            return ds;
        }


        



    }
}
