﻿using System;
using System.Data;
using System.Data.OleDb;


namespace PhonicsClassLibrary
{
    public interface IUtilizador
    {
        /// <summary>
        /// Método para validar utilizadores no sistema
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos</pre>
        /// <post>Valida o utilizador</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum Validar(string username, string pass);

        /// <summary>
        /// Método para actualizar a comissao dos operadores
        /// </summary>
        /// <param name="username">Username do utilizador</param>
        /// <param name="pass">Password do utilizador</param>
        /// <param name="comissao">Comissao associada ao tarifario</param>
        /// <pre>Este método recebe o username e password do utilizador
        /// válidos</pre>
        /// <post>Actualizar a comissao dos operadores</post>
        /// <returns>Retorna o estado da operação</returns>
        PhonicsStatusEnum ActualizarComissao(string username, string pass,
            float comissao);
    }

   
    internal class Utilizador : IUtilizador
    {
        public Utilizador()
        {
        }
        
        public PhonicsStatusEnum Validar(string username, string pass)
        {
            return LigaDB.Validar(username, pass);
        }

        public PhonicsStatusEnum ActualizarComissao(string username, string pass, float comissao)
        {
            PhonicsStatusEnum status = PhonicsStatusEnum.OK;

            OleDbConnection conn = null;


            try
            {
                //
                // criar objecto de conexão à base de dados e abrir a conexão
                //
                conn = new OleDbConnection(LigaDB.CONN);
                conn.Open();

                //
                // validar o utilizador
                //
                status = LigaDB.Validar(conn, null, username, pass);
                if (status != PhonicsStatusEnum.OK)
                    return status;

                //
                // criar comando SQL a executar
                //
                //ACTUALIZAR A COMISSÃO, É SOMADA A NOVA COMISSÃO À EXISTENTE
                string sqlCmd = "UPDATE Utilizador SET Comissao=Comissao+? WHERE username=? AND pass=?";

                OleDbCommand cmd = new OleDbCommand(sqlCmd, conn);

                cmd.Parameters.AddWithValue("Comissao", comissao);
                cmd.Parameters.AddWithValue("username", username);
                cmd.Parameters.AddWithValue("pass", pass);



                //
                // executar o comando 
                //
                int insRows = cmd.ExecuteNonQuery();

                //
                // verificar o resultado
                //
                if (insRows == 0)
                    // não inseriu registo
                    status = PhonicsStatusEnum.NOT_OK;

            }
            catch (OleDbException ex)
            {
                //
                // tratar a excepção!!!!
                //
                System.Console.WriteLine("EXCEPÇÃO no método <Update> da classe 'Utilizador': " + ex.Message);
                status = PhonicsStatusEnum.ERROR;
            }
            finally
            {
                //
                // fechar a conexão
                //
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }
            return status;
        }
    }
}
